from ._exceptions import (
    InvalidFilterExtractorFormatError,
    InvalidIntentRouterFormatError,
)
from ._config import CONFIG

CONFIG_INTENT_ROUTER = CONFIG["intent_router"]
CONFIG_FILTER_EXTRACTOR = CONFIG["filter_extractor"]


def validate_intent_router_format(query, routes, allow_none):
    validate_intent_router_query(query)
    validate_intent_router_allow_none(allow_none)
    validate_routes(routes)


def validate_filter_extractor_format(query, field, required):
    validate_filter_extractor_query(query)
    validate_filter_extractor_required(required)
    validate_field(field)


def validate_intent_router_allow_none(allow_none):
    if not isinstance(allow_none, bool):
        raise InvalidIntentRouterFormatError(
            "The 'allow_none' parameter must be a boolean"
        )


def validate_intent_router_query(query):
    if not isinstance(query, str):
        raise InvalidIntentRouterFormatError("The 'query' must be a string")
    max_query_length = CONFIG_INTENT_ROUTER["max_query_length"]
    if len(query) > max_query_length:
        raise InvalidIntentRouterFormatError("Too long query")


def validate_routes(routes):
    if not isinstance(routes, list):
        raise InvalidIntentRouterFormatError("The 'routes' must be a list")
    if len(routes) == 0:
        raise InvalidIntentRouterFormatError("At least one route must be provided")
    max_routes_num = CONFIG_INTENT_ROUTER["max_routes_num"]
    if len(routes) > max_routes_num:
        raise InvalidIntentRouterFormatError(
            f"Too many routes ({len(routes)} routes > {max_routes_num} routes)"
        )
    for route_index, route in enumerate(routes):
        validate_route(route, route_index)


def validate_route(route_dict: dict, route_index: int):
    required_keys = ["name", "description"]
    optional_keys = ["utterances"]

    for key in required_keys:
        if key not in route_dict:
            raise InvalidIntentRouterFormatError(
                f"routes[{route_index}]: Missing required key: '{key}'"
            )
        if not isinstance(route_dict[key], str):
            raise InvalidIntentRouterFormatError(
                f"routes[{route_index}]: Value for key '{key}' must be a string"
            )

    max_route_name_length = CONFIG_INTENT_ROUTER["max_route_name_length"]
    if len(route_dict["name"]) > max_route_name_length:
        raise InvalidIntentRouterFormatError(f"Too long routes[{route_index}]'s name")
    max_route_description_length = CONFIG_INTENT_ROUTER["max_route_description_length"]
    if len(route_dict["description"]) > max_route_description_length:
        raise InvalidIntentRouterFormatError(
            f"Too long routes[{route_index}]'s description"
        )

    if "utterances" in route_dict:
        if not isinstance(route_dict["utterances"], list) or not all(
            isinstance(item, str) for item in route_dict["utterances"]
        ):
            raise InvalidIntentRouterFormatError(
                f"routes[{route_index}]: Value for key 'utterances' must be a list of strings"
            )
        max_route_utterances_num = CONFIG_INTENT_ROUTER["max_route_utterances_num"]
        if len(route_dict["utterances"]) > max_route_utterances_num:
            raise InvalidIntentRouterFormatError(
                f"Too many utterances in routes[{route_index}]"
            )

        max_route_utterance_length = CONFIG_INTENT_ROUTER["max_route_utterance_length"]
        for utterance_index, utterance in enumerate(route_dict["utterances"]):
            if len(utterance) > max_route_utterance_length:
                raise InvalidIntentRouterFormatError(
                    f"Too long utterances[{utterance_index}] of routes[{route_index}]"
                )

    for key in route_dict.keys():
        if key not in required_keys and key not in optional_keys:
            raise InvalidIntentRouterFormatError(f"Unexpected key: {key}")


def validate_filter_extractor_query(query):
    if not isinstance(query, str):
        raise InvalidFilterExtractorFormatError("The query must be a string.")
    max_query_length = CONFIG_FILTER_EXTRACTOR["max_query_length"]
    if len(query) > max_query_length:
        raise InvalidFilterExtractorFormatError("Too long query.")


def validate_filter_extractor_required(required):
    if not isinstance(required, bool):
        raise InvalidFilterExtractorFormatError(
            "The 'required' parameter must be a boolean."
        )


def validate_field(field_dict):
    if not isinstance(field_dict, dict):
        raise InvalidFilterExtractorFormatError("The 'field' must be a dictionary.")
    required_keys = {"name", "description", "type"}
    optional_key = "values"
    allowed_types = {"string", "number", "list", "date"}
    allowed_types_for_values = {"list", "string"}

    for key in required_keys:
        if key not in field_dict:
            raise InvalidFilterExtractorFormatError(f"Missing required key: {key}.")

    for key in field_dict:
        if key not in required_keys and key != optional_key:
            raise InvalidFilterExtractorFormatError(f"Unexpected key: {key}.")

    if not isinstance(field_dict["name"], str):
        raise InvalidFilterExtractorFormatError("The value of 'name' must be a string.")

    max_field_name_legnth = CONFIG_FILTER_EXTRACTOR["max_field_name_legnth"]
    if len(field_dict["name"]) > max_field_name_legnth:
        raise InvalidFilterExtractorFormatError("Too long field name")

    if not isinstance(field_dict["description"], str):
        raise InvalidFilterExtractorFormatError(
            "The value of 'description' must be a string."
        )

    max_field_description_length = CONFIG_FILTER_EXTRACTOR[
        "max_field_description_length"
    ]
    if len(field_dict["description"]) > max_field_description_length:
        raise InvalidFilterExtractorFormatError("Too long field description")

    if optional_key in field_dict:

        if not isinstance(field_dict["type"], str):
            raise InvalidFilterExtractorFormatError(
                f"The value of 'type' must be one of {allowed_types}."
            )
        if field_dict["type"] not in allowed_types:
            raise InvalidFilterExtractorFormatError(
                f"The value of 'type' must be one of {allowed_types}."
            )

        if not field_dict["type"] in allowed_types_for_values:
            raise InvalidFilterExtractorFormatError(
                f"The 'value' parameter is not allowed when the field type is '{field_dict['type']}'."
            )
        if not isinstance(field_dict[optional_key], list):
            raise InvalidFilterExtractorFormatError(
                f"The value of '{optional_key}' must be a list of strings."
            )
        if not all(isinstance(item, str) for item in field_dict[optional_key]):
            raise InvalidFilterExtractorFormatError(
                f"All elements in the '{optional_key}' list must be strings."
            )

        if (
            len(field_dict[optional_key])
            > CONFIG_FILTER_EXTRACTOR["max_field_values_num"]
        ):
            raise InvalidFilterExtractorFormatError(
                f"Too many values ({len(field_dict[optional_key])} values > {CONFIG_FILTER_EXTRACTOR['max_field_values_num']} values)"
            )

        for value_index, value in enumerate(field_dict[optional_key]):
            max_field_value_length = CONFIG_FILTER_EXTRACTOR["max_field_value_length"]
            if len(value) > max_field_value_length:
                raise InvalidFilterExtractorFormatError(
                    f"Too long values[{value_index}] string"
                )



import json
from ._exceptions import FilterExtractorError
from ._rest_adapter import Response

INDENT = 2


class Filter:
    def __init__(self, operator, value: str):
        self.type = "Filter"
        self.operator = operator
        self.value = value

    def to_dict(self):
        return {
            "type": self.type,
            "operator": self.operator,
            "value": self.value,
        }

    def __repr__(self):
        return json.dumps(self.to_dict(), indent=INDENT)

    def __str__(self):
        return json.dumps(self.to_dict(), indent=INDENT)


class ConjunctedFilter:
    def __init__(self, conjunction, filters_dict):
        self.type = "ConjunctedFilter"
        self.conjunction = conjunction
        self.filters = self._get_filters(filters_dict)

    def _get_filters(self, filters_dict):
        filters = []
        for filter in filters_dict:
            if filter["type"] == "Filter":
                filters.append(Filter(filter["operator"], filter["value"]))
            elif filter["type"] == "ConjunctedFilter":
                filters.append(
                    ConjunctedFilter(filter["conjunction"], filter["filters"])
                )
            else:
                raise FilterExtractorError("Invalid filter type")
        return filters

    def to_dict(self):
        return {
            "type": self.type,
            "conjunction": self.conjunction,
            "filters": [filter.to_dict() for filter in self.filters],
        }

    def __repr__(self):
        return json.dumps(self.to_dict(), indent=INDENT)

    def __str__(self):
        return json.dumps(self.to_dict(), indent=INDENT)


class FilterExtractorResponse:
    def __init__(self, response: Response):
        self.result = self._get_result(response.data)

    def _get_result(self, data):
        result_dict = data["result"]
        if result_dict == {}:
            return {}

        if result_dict["type"] == "ConjunctedFilter":
            result = ConjunctedFilter(
                conjunction=result_dict["conjunction"],
                filters_dict=result_dict["filters"],
            )
        elif result_dict["type"] == "Filter":
            result = Filter(
                operator=result_dict["operator"],
                value=result_dict["value"],
            )
        else:
            raise FilterExtractorError("Invalid filter type")

        return result

    def to_dict(self):
        return {"result": self.result.to_dict()}

    def __repr__(self):
        return json.dumps(self.to_dict(), indent=INDENT)

    def __str__(self):
        return json.dumps(self.to_dict(), indent=INDENT)



TOKEN_STRING_LENGTH_RATIO = 6.5
CONFIG = {
    "intent_router": {
        "max_query_length": 50 * TOKEN_STRING_LENGTH_RATIO,
        "max_routes_num": 5,
        "max_route_name_length": 10 * TOKEN_STRING_LENGTH_RATIO,
        "max_route_description_length": 100 * TOKEN_STRING_LENGTH_RATIO,
        "max_route_utterances_num": 50,
        "max_route_utterance_length": 50 * TOKEN_STRING_LENGTH_RATIO,
    },
    "filter_extractor": {
        "max_query_length": 50 * TOKEN_STRING_LENGTH_RATIO,
        "max_field_name_legnth": 10 * TOKEN_STRING_LENGTH_RATIO,
        "max_field_description_length": 50 * TOKEN_STRING_LENGTH_RATIO,
        "max_field_values_num": 10,
        "max_field_value_length": 50 * TOKEN_STRING_LENGTH_RATIO,
    },
}



import logging
from typing import Union, List

from ._rest_adapter import RestAdapter, AsyncRestAdapter
from ._validate import validate_intent_router_format, validate_filter_extractor_format
from .intent_router import IntentRouterResponse, Routes
from .filter_extractor import FilterExtractorResponse
from ._exceptions import APIError, APIConnectionError


class BaseClient:
    def __init__(
        self,
        api_key: str = "",
        ver: str = "v1",
        ssl_verify: bool = True,
        logger: logging.Logger = None,
        adapter_class=None,
        exception_class=None,
    ):
        self._rest_adapter = adapter_class(
            hostname="api.sineps.io",
            api_key=api_key,
            ver=ver,
            ssl_verify=ssl_verify,
            logger=logger,
            exception_class=exception_class,
        )
        self._check_api_key()

    def _check_api_key(self):
        if not self._rest_adapter._api_key:
            raise APIError("SINEPS API key is required")

    def exec_intent_router(
        self,
        query: str,
        routes: Union[Routes, List[dict]] = [],
        allow_none: bool = False,
    ):
        if isinstance(routes, Routes):
            routes = routes.to_dict()
        validate_intent_router_format(query, routes, allow_none)
        data = {"query": query, "routes": routes, "allow_none": allow_none}
        return data

    def exec_filter_extractor(
        self, query: str, field: dict = {}, required: bool = False
    ):
        validate_filter_extractor_format(query, field, required)
        data = {"query": query, "field": field, "required": required}
        return data


class Client(BaseClient):
    def __init__(
        self,
        api_key: str = "",
        ver: str = "v1",
        ssl_verify: bool = True,
        logger: logging.Logger = None,
    ):
        super().__init__(
            api_key,
            ver,
            ssl_verify,
            logger,
            RestAdapter,
            exception_class=APIConnectionError,
        )

    def exec_intent_router(
        self,
        query: str,
        routes: Union[Routes, List[dict]] = [],
        allow_none: bool = False,
    ):
        data = super().exec_intent_router(query, routes, allow_none)
        result = self._rest_adapter.post("/intent-router", data=data)
        return IntentRouterResponse(result, routes)

    def exec_filter_extractor(
        self, query: str, field: dict = {}, required: bool = False
    ):
        data = super().exec_filter_extractor(query, field, required)
        result = self._rest_adapter.post("/filter-extractor", data=data)
        return FilterExtractorResponse(result)


class AsyncClient(BaseClient):
    def __init__(
        self,
        api_key: str = "",
        ver: str = "v1",
        ssl_verify: bool = True,
        logger: logging.Logger = None,
    ):
        super().__init__(
            api_key,
            ver,
            ssl_verify,
            logger,
            AsyncRestAdapter,
            exception_class=APIConnectionError,
        )

    async def exec_intent_router(
        self,
        query: str,
        routes: Union[Routes, List[dict]] = [],
        allow_none: bool = False,
    ):
        data = super().exec_intent_router(query, routes, allow_none)
        result = await self._rest_adapter.post("/intent-router", data=data)
        return IntentRouterResponse(result, routes)

    async def exec_filter_extractor(
        self, query: str, field: dict = {}, required: bool = False
    ):
        data = super().exec_filter_extractor(query, field, required)
        result = await self._rest_adapter.post("/filter-extractor", data=data)
        return FilterExtractorResponse(result)



def handle_error_message(is_success: bool, data_out: dict):
    if "detail" in data_out and not is_success:
        return data_out["detail"]
    elif is_success:
        return "Success"
    return None


def add_index_to_dictionary_list(dict_list: list):
    new_dict_list = []
    for i, item in enumerate(dict_list):
        new_item = item.copy()
        new_item["index"] = i
        new_dict_list.append(new_item)
    return new_dict_list



import requests
import aiohttp
import requests.packages
from typing import List, Dict
import logging
from json import JSONDecodeError

from ._exceptions import (
    RestAdapterError,
    InternalServerError,
    BadRequestError,
    TooManyRequestsError,
    UnauthorizedAPIKeyError,
    PaymentRequiredError,
)


from ._utils import handle_error_message


class Response:
    def __init__(self, status_code: int, message: str = "", data: List[Dict] = None):
        self.status_code = int(status_code)
        self.message = str(message)
        self.data = data if data else []


class BaseRestAdapter:
    def __init__(
        self,
        hostname: str,
        api_key: str = "",
        ver: str = "v1",
        ssl_verify: bool = True,
        logger: logging.Logger = None,
    ):
        self._logger = logger or logging.getLogger(__name__)
        self.url = f"https://{hostname}/{ver}"
        self._api_key = api_key
        self._ssl_verify = ssl_verify

    def _define_exception_class(self, exception_class, base_exception_class):
        if exception_class:
            return exception_class
        return base_exception_class

    def _build_log_lines(self, http_method: str, endpoint: str, ep_params: Dict):
        full_url = self.url + endpoint
        log_line_pre = f"method={http_method}, url={full_url}, params={ep_params}"
        log_line_post = ", ".join(
            (log_line_pre, "success={}, status_code={}, message={}")
        )
        return full_url, log_line_pre, log_line_post

    def _log_and_raise_exception(
        self, log_line_post, success, status_code, message, exception
    ):
        log_line = log_line_post.format(success, status_code, message)
        if success:
            self._logger.debug(msg=log_line)
        else:
            self._logger.debug(msg=log_line)

            if status_code == 400:
                raise BadRequestError(
                    f"{status_code} - {message}",
                    status_code=status_code,
                    message=message,
                )
            elif status_code == 401:
                raise UnauthorizedAPIKeyError(
                    f"{status_code} - {message}",
                    status_code=status_code,
                    message=message,
                )
            elif status_code == 402:
                raise PaymentRequiredError(
                    f"{status_code} - {message}",
                    status_code=status_code,
                    message=message,
                )
            elif status_code == 429:
                raise TooManyRequestsError(
                    f"{status_code} - {message}",
                    status_code=status_code,
                    message=message,
                )
            elif status_code == 500:
                raise InternalServerError(
                    f"{status_code} - {message}",
                    status_code=status_code,
                    message=message,
                )
            raise exception


class RestAdapter(BaseRestAdapter):
    def __init__(
        self, hostname, api_key, ver, ssl_verify, logger, exception_class=None
    ):
        super().__init__(hostname, api_key, ver, ssl_verify, logger)
        self._exception_class = self._define_exception_class(
            exception_class, RestAdapterError
        )

    def _do(
        self, http_method: str, endpoint: str, ep_params: Dict = None, data: Dict = None
    ):
        full_url, log_line_pre, log_line_post = self._build_log_lines(
            http_method, endpoint, ep_params
        )
        headers = {
            "Content-Type": "application/json",
            "api-key": self._api_key,
        }

        try:
            self._logger.debug(msg=log_line_pre)
            response = requests.request(
                method=http_method,
                url=full_url,
                verify=self._ssl_verify,
                headers=headers,
                params=ep_params,
                json=data,
            )
        except requests.exceptions.RequestException as e:
            self._logger.error(msg=(str(e)))
            raise self._exception_class("Request failed") from e

        try:
            data_out = response.json()
        except (ValueError, JSONDecodeError) as e:
            self._logger.error(msg=log_line_post.format(False, None, e))
            raise self._exception_class(f"{response.status_code}") from e

        is_success = 299 >= response.status_code >= 200
        message = handle_error_message(is_success, data_out)
        self._log_and_raise_exception(
            log_line_post,
            is_success,
            response.status_code,
            message,
            self._exception_class,
        )

        return Response(response.status_code, message=message, data=data_out)

    def get(self, endpoint: str, ep_params: Dict = None) -> Response:
        return self._do(http_method="GET", endpoint=endpoint, ep_params=ep_params)

    def post(
        self, endpoint: str, ep_params: Dict = None, data: Dict = None
    ) -> Response:
        return self._do(
            http_method="POST", endpoint=endpoint, ep_params=ep_params, data=data
        )

    def delete(
        self, endpoint: str, ep_params: Dict = None, data: Dict = None
    ) -> Response:
        return self._do(
            http_method="DELETE", endpoint=endpoint, ep_params=ep_params, data=data
        )


class AsyncRestAdapter(BaseRestAdapter):

    def __init__(
        self, hostname, api_key, ver, ssl_verify, logger, exception_class=None
    ):
        super().__init__(hostname, api_key, ver, ssl_verify, logger)
        self._exception_class = self._define_exception_class(
            exception_class, RestAdapterError
        )

    async def _do(
        self, http_method: str, endpoint: str, ep_params: Dict = None, data: Dict = None
    ):
        full_url, log_line_pre, log_line_post = self._build_log_lines(
            http_method, endpoint, ep_params
        )
        headers = {
            "Content-Type": "application/json",
            "api-key": self._api_key,
        }

        try:
            self._logger.debug(msg=log_line_pre)
            async with aiohttp.ClientSession() as session:
                async with session.request(
                    method=http_method,
                    url=full_url,
                    ssl=self._ssl_verify,
                    headers=headers,
                    params=ep_params,
                    json=data,
                ) as response:
                    try:
                        data_out = await response.json()
                    except (ValueError, JSONDecodeError) as e:
                        self._logger.error(msg=log_line_post.format(False, None, e))
                        raise self._exception_class(f"{response.status_code}") from e

                    is_success = 299 >= response.status >= 200
                    message = handle_error_message(is_success, data_out)
                    self._log_and_raise_exception(
                        log_line_post,
                        is_success,
                        response.status,
                        message,
                        self._exception_class(f"{response.status} - {message}"),
                    )

                    return Response(response.status, message=message, data=data_out)
        except aiohttp.ClientError as e:
            self._logger.error(msg=(str(e)))
            raise self._exception_class("Request failed") from e

    async def get(self, endpoint: str, ep_params: Dict = None) -> Response:
        return await self._do(http_method="GET", endpoint=endpoint, ep_params=ep_params)

    async def post(
        self, endpoint: str, ep_params: Dict = None, data: Dict = None
    ) -> Response:
        return await self._do(
            http_method="POST", endpoint=endpoint, ep_params=ep_params, data=data
        )

    async def delete(
        self, endpoint: str, ep_params: Dict = None, data: Dict = None
    ) -> Response:
        return await self._do(
            http_method="DELETE", endpoint=endpoint, ep_params=ep_params, data=data
        )



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="sineps",
    version="0.0.18",
    author="Ceres Technologies",
    author_email="infosec@sineps.io",
    description="The official Python library for the sineps API",
    long_description=long_description,
    long_description_content_type="text/markdown",
    packages=setuptools.find_packages(),
    python_requires=">=3.6",
    license=" ",
    url=" ",
    install_requires=[
        "requests>=2.24.0",
        "aiohttp>=3.6.2",
    ],
)



import json
from typing import List

from ._rest_adapter import Response


INDENT = 2


class Route:
    def __init__(
        self,
        name: str,
        description: str,
        utterances=[],
        index=None,
    ):
        self.index = index
        self.name = name
        self.description = description
        self.utterances = utterances

    def to_str_dict(self):
        if self.index is None:
            return {
                "name": self.name,
                "description": self.description,
                "utterances": self.utterances,
            }
        else:
            return {
                "index": self.index,
                "name": self.name,
                "description": self.description,
                "utterances": self.utterances,
            }

    def to_dict(self):
        return {
            "name": self.name,
            "description": self.description,
            "utterances": self.utterances,
        }

    def __repr__(self, indent=INDENT):
        return json.dumps(self.to_dict(), indent=indent)

    def __str__(self, indent=INDENT):
        return json.dumps(self.to_str_dict(), indent=indent)


class Routes:
    def __init__(self, routes: List[Route]):
        self.routes = routes

    def to_dict(self):
        return [route.to_dict() for route in self.routes]

    def __repr__(self, indent=INDENT):
        return json.dumps(self.to_dict(), indent=indent)

    def __str__(self, indent=INDENT):
        return json.dumps(self.to_dict(), indent=indent)


class IntentRouterResponse:
    def __init__(self, response: Response, all_routes: List[dict]):
        self.result = self._get_result(response.data, all_routes)

    def _get_result_route_indices(self, data):
        routes = data["result"]["routes"]
        if len(routes) == 0:
            return []
        else:
            return [route["index"] for route in routes]

    def _get_result(self, data, all_routes):
        if isinstance(all_routes, Routes):
            all_routes = all_routes.to_dict()
        result_routes_indices = self._get_result_route_indices(data)
        result = Routes(
            routes=[Route(**all_routes[i], index=i) for i in result_routes_indices]
        )
        return result

    def to_dict(self):

        return {"result": self.result.to_dict()}

    def __repr__(self):
        return json.dumps(self.to_dict(), indent=INDENT)

    def __str__(self):
        return json.dumps(self.to_dict(), indent=INDENT)



class APIError(Exception):
    pass


class IntentRouterError(APIError):
    def __init__(self, message: str):
        super().__init__(message)
        self.message = message


class InvalidIntentRouterFormatError(IntentRouterError):
    pass


class FilterExtractorError(APIError):
    def __init__(self, message: str):
        super().__init__(message)
        self.message = message


class InvalidFilterExtractorFormatError(FilterExtractorError):
    pass


class APIStatusError(APIError):
    def __init__(self, text: str, status_code: int, message: str):
        super().__init__(text)
        self.status_code = status_code
        self.message = message


class APIConnectionError(APIError):
    pass


class RestAdapterError(APIError):
    pass


class InternalServerError(APIStatusError):
    pass


class BadRequestError(APIStatusError):
    pass


class TooManyRequestsError(APIStatusError):
    pass


class UnauthorizedAPIKeyError(APIStatusError):
    pass


class PaymentRequiredError(APIStatusError):
    pass
